package com.imcode.forum;

import org.apache.commons.lang.NullArgumentException;

import java.util.*;

public class SimpleForumPost implements ForumPost {

    public static final Comparator<ForumPost> DATETIME_COMPARATOR = new Comparator<ForumPost>() {
        public int compare(ForumPost o1, ForumPost o2) {
            return o1.getDateTime().compareTo(o2.getDateTime()) ;
        }
    };

    private String title = "";
    private String body = "";
    private ForumUser user = new SimpleForumUser("", "", "", "");
    private Date dateTime = new Date();
    private int viewCount;
    private SortedSet<ForumPost> replies = new TreeSet<ForumPost>(DATETIME_COMPARATOR);
    private boolean hidden;

    public SimpleForumPost() {}

    public SimpleForumPost(String title, String body, ForumUser user) {
        setTitle(title);
        setBody(body);
        setUser(user);
    }

    public int getViewCount() {
        return viewCount ;
    }

    public String createReply(ForumPost forumPost) {
        replies.add(forumPost) ;
        return forumPost.getId() ;
    }

    public void setTitle(String title) {
        if (null == title) {
            throw new NullArgumentException("title") ;
        }
        this.title = title;
    }

    public void setBody(String body) {
        if (null == body) {
            throw new NullArgumentException("body") ;
        }
        this.body = body;
    }

    public void setUser(ForumUser user) {
        this.user = user;
    }

    public void setDateTime(Date dateTime) {
        if (null == dateTime) {
            throw new NullArgumentException("dateTime") ;
        }
        this.dateTime = (Date) dateTime.clone();
    }

    public String getId() {
        return null ;
    }

    public ForumThread getThread() {
        return null ;
    }

    public ForumPost getParentPost() {
        return null ;
    }

    public String getTitle() {
        return title ;
    }

    public Date getDateTime() {
        return (Date) dateTime.clone() ;
    }

    public String getBody() {
        return body ;
    }

    public boolean isHidden() {
        return hidden;
    }

    public void hide(boolean hidden) {
        this.hidden = hidden ;
    }

    public ForumUser getUser() {
        return user ;
    }

    public SortedSet<ForumPost> getReplies() {
        return Collections.unmodifiableSortedSet(replies);
    }

    public void incrementViewCount() {
        viewCount++;
    }

}
