package com.imcode.forum.sql;

import com.imcode.db.handlers.RowTransformer;
import com.imcode.forum.SimpleForumUser;

import java.sql.ResultSet;
import java.sql.SQLException;

class SqlPostFactory implements RowTransformer {
    static final String POST_COLUMNS = "forum_posts.post_id,"
                                       + " forum_posts.thread_id,"
                                       + " forum_posts.parent_post_id,"
                                       + " forum_posts.title,"
                                       + " forum_posts.body,"
                                       + " forum_posts.datetime,"
                                       + " forum_posts.view_count,"
                                       + " forum_posts.user_name,"
                                       + " forum_posts.user_alias,"
                                       + " forum_posts.user_email_address,"
                                       + " forum_posts.userdata,"
                                       + " forum_posts.hidden" ;
    private SqlRepository repository;
    private final SqlThread thread;

    SqlPostFactory(SqlRepository repository, SqlThread thread) {
        this.repository = repository;
        this.thread = thread;
    }

    public Object createObjectFromResultSetRow(ResultSet resultSet) throws SQLException {
        String postId = resultSet.getString("post_id");
        String threadId = resultSet.getString("thread_id") ;
        SqlPost forumPost = new SqlPost(postId, repository, threadId, thread);
        forumPost.setParentPostId(resultSet.getString("parent_post_id")) ;
        forumPost.setTitle(resultSet.getString("title")) ;
        forumPost.setBody(resultSet.getString("body"));
        forumPost.setDateTime(resultSet.getTimestamp("datetime")) ;
        forumPost.setViewCount(resultSet.getInt("view_count")) ;
        String name = resultSet.getString("user_name");
        String alias = resultSet.getString("user_alias");
        String emailAddress = resultSet.getString("user_email_address");
        String data = resultSet.getString("userdata");
        forumPost.setUser(new SimpleForumUser(name, alias, emailAddress, data));
        forumPost.setHidden(resultSet.getBoolean("hidden"));
        return forumPost ;
    }

    public Class getClassOfCreatedObjects() {
        return SqlPost.class ;
    }
}
