package com.imcode.forum.sql;

import com.imcode.db.Database;
import com.imcode.db.DatabaseCommand;
import com.imcode.db.DatabaseConnection;
import com.imcode.db.DatabaseException;
import com.imcode.db.jdbc.ConnectionWrapper;
import org.apache.commons.lang.NotImplementedException;
import org.apache.ddlutils.Platform;
import org.apache.ddlutils.PlatformFactory;

import javax.sql.DataSource;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

public class TableCreatingDatabase implements Database {

    private final Database database;
    private org.apache.ddlutils.model.Database ddlDatabase;

    private static final String SQL_STATE__MISSING_TABLE = "42S02";

    public TableCreatingDatabase(Database database,
                                 org.apache.ddlutils.model.Database ddlDatabase) {
        this.database = database ;
        this.ddlDatabase = ddlDatabase ;
    }

    public Object execute(DatabaseCommand databaseCommand) throws DatabaseException {
        try {
            return database.execute(databaseCommand) ;
        } catch (DatabaseException dbe) {
            for ( Throwable t = dbe ; null != t; t = t.getCause() ) {
                if (t instanceof SQLException ) {
                    for (SQLException se = (SQLException) t; null != se; se = se.getNextException() ) {
                        if (SQL_STATE__MISSING_TABLE.equals(se.getSQLState())) {
                            createTables();
                            return database.execute(databaseCommand);
                        }
                    }
                }
            }
            throw dbe ;
        }
    }

    public Object executeCommand(DatabaseCommand databaseCommand) throws DatabaseException {
        return execute(databaseCommand) ;
    }

    public void createTables() {
        database.execute(new DatabaseCommand() {
            public Object executeOn(final DatabaseConnection connection) throws DatabaseException {
                Platform platform = PlatformFactory.createNewPlatformInstance(new SingleUnclosableConnectionMockDataSource(connection.getConnection()));
                platform.createTables(ddlDatabase, false, true);
                return null ;
            }
        }) ;
    }

    private static class SingleUnclosableConnectionMockDataSource implements DataSource {

        private final Connection uncloseableConnection;

        SingleUnclosableConnectionMockDataSource(Connection connection) {
            uncloseableConnection = new ConnectionWrapper(connection) {
                public void close() {}
            };
        }

        public Connection getConnection() throws SQLException {
            return uncloseableConnection ;
        }

        public Connection getConnection(String username, String password) throws SQLException {
            throw new NotImplementedException(".getConnection");
        }

        public PrintWriter getLogWriter() throws SQLException {
            throw new NotImplementedException(".getLogWriter");
        }

        public void setLogWriter(PrintWriter out) throws SQLException {
            throw new NotImplementedException(".setLogWriter");
        }

        public void setLoginTimeout(int seconds) throws SQLException {
            throw new NotImplementedException(".setLoginTimeout");
        }

        public int getLoginTimeout() throws SQLException {
            throw new NotImplementedException(".getLoginTimeout");
        }
    }
}
